<?php
namespace BotGuard;
/*
 * botguard_api.php
 * This file is an implementation of the BotGuard Partner API
 *
 * Copyright (c) 2020 Dennis Prochko <wolfsoft@mail.ru>
*/

class BotGuard {

	private $api_key;

	/**
	 * Constructor. Initialize the instance of the BotGuard class.
	 *
	 * @param string $api_key BotGuard API Key to access all functions
	 */
	public function __construct(string $api_key) {
		$this->api_key = $api_key;
	}

	/**
	 * Get a list of all registered domains in your BotGuard account.
	 *
	 * @returns An array of JSON objects
	 */
	public function getDomains() {
		$result = $this->doRequest(
			'GET',
			"https://api.botguard.net/en/website"
		);
		if (!$result) {
			throw new \Exception('There are no domains registered.');
		}
		if (property_exists($result, 'status') && $result->status == 'error') {
			throw new \Exception($result->message);
		}

		return $result;
	}

	/**
	 * Add domain into your BotGuard account.
	 *
	 * @param string $domain Domain name
	 * @options array core rules mitigation options
	 */
	public function addDomain(string $domain, array $options = []) {
		$result = $this->doRequest(
			'POST',
			'https://api.botguard.net/en/website', [
				'url' => trim($domain)
			] + $options
		);
		if (!$result) {
			throw new \Exception('Domain registration error. Please go back and try again.');
		}
		if (property_exists($result, 'status') && $result->status == 'error') {
			throw new \Exception($result->message);
		}
	}

	/**
	 * Get an information about domain in your BotGuard account.
	 *
	 * @param string $domain Domain name
	 *
	 * @returns JSON object
	 */
	public function getDomain(string $domain) {
		$d = urlencode(trim($domain));
		$result = $this->doRequest(
			'GET',
			"https://api.botguard.net/en/website/$d"
		);
		if (!$result) {
			throw new \Exception('Domain is not registered.');
		}
		if (property_exists($result, 'status') && $result->status == 'error') {
			throw new \Exception($result->message);
		}

		return $result;
	}

	/**
	 * Delete domain from your BotGuard account.
	 *
	 * @param string $domain Domain name
	 */
	public function deleteDomain(string $domain) {
		$d = urlencode(trim($domain));
		$result = $this->doRequest(
			'DELETE',
			"https://api.botguard.net/en/website/$d"
		);
		if (!$result) {
			throw new \Exception('Domain deregistration error. Please go back and try again.');
		}
		if (property_exists($result, 'status') && $result->status == 'error') {
			throw new \Exception($result->message);
		}
	}

	/**
	 * Suspend domain in your BotGuard account.
	 *
	 * @param string $domain Domain name
	 */
	public function suspendDomain(string $domain) {
		$d = urlencode(trim($domain));
		$result = $this->doRequest(
			'PUT',
			"https://api.botguard.net/en/website/$d", [
				'status' => 'paused'
			]
		);
		if (!$result) {
			throw new \Exception('Domain suspending error. Please go back and try again.');
		}
		if (property_exists($result, 'status') && $result->status == 'error') {
			throw new \Exception($result->message);
		}
	}

	/**
	 * Unsuspend (bring online) domain in your BotGuard account.
	 *
	 * @param string $domain Domain name
	 */
	public function unsuspendDomain(string $domain) {
		$d = urlencode(trim($domain));
		$result = $this->doRequest(
			'PUT',
			"https://api.botguard.net/en/website/$d", [
				'status' => 'online'
			]
		);
		if (!$result) {
			throw new \Exception('Domain unsuspending error. Please go back and try again.');
		}
		if (property_exists($result, 'status') && $result->status == 'error') {
			throw new \Exception($result->message);
		}
	}

	/**
	 * Get a link to the Events page for iframe embedding.
	 *
	 * @param string $domain Domain name
	 *
	 * @returns An URL to the page
	 */
	public function getEventsPageLink(string $domain, string $lang = 'en') {
		$d = urlencode(trim($domain));
		$timestamp = time();
		$dtoken = base64_encode($d . ';' . $timestamp . ';' . md5($d . ';' . $timestamp . ';' . $this->api_key));
		return "https://api.botguard.net/$lang/events/$d?dtoken=$dtoken";
	}

	/**
	 * Get a link to the Rules Editor page for iframe embedding.
	 *
	 * @param string $domain Domain name
	 *
	 * @returns An URL to the page
	 */
	public function getRulesEditorPageLink(string $domain, string $lang = 'en') {
		$d = urlencode(trim($domain));
		$timestamp = time();
		$dtoken = base64_encode($d . ';' . $timestamp . ';' . md5($d . ';' . $timestamp . ';' . $this->api_key));
		return "https://api.botguard.net/$lang/settings/$d/edit?dtoken=$dtoken";
	}


	/**
	 * Make a generic https request to the BotGuard API endpoint.
	 *
	 * @param string $method HTTP method (GET, POST, ...)
	 * @param string $url URL to make request to
	 * @param string $params An array of "name=value" parameters to pass into the request (optional)
	 *
	 * @returns JSON object
	 */
	private function doRequest(string $method, string $url, array $params = null) {
		$http = [
			'protocol_version' => '1.1',
			'header' => "Accept: application/json\r\nAuthorization: Bearer {$this->api_key}\r\n",
			'method' => $method,
			'ignore_errors' => true
		];
		if ($params !== null) {
			$http['header'] .= "Content-type: application/x-www-form-urlencoded\r\n";
			$http['content'] = http_build_query($params);
		}
		$context = stream_context_create([
			'http' => $http,
			'ssl' => [
				'peer_name' => 'api.botguard.net',
				'verify_depth' => 3,
				'SNI_enabled' => true
			]
		]);
		return @json_decode(@file_get_contents($url, false, $context));
	}

}
